package com.order.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.model.ResponseData;
import com.hfkj.model.UserSessionObject;
import com.hfkj.model.order.OrderChildModel;
import com.hfkj.model.order.OrderModel;
import com.hfkj.service.order.BsOrderService;
import com.hfkj.service.pay.HuiPayService;
import com.hfkj.sysenum.UserStatusEnum;
import com.hfkj.sysenum.order.OrderChildProductTypeEnum;
import com.hfkj.sysenum.order.OrderPayChannelEnum;
import com.hfkj.sysenum.order.OrderPayTypeEnum;
import com.hfkj.sysenum.order.OrderStatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @className: OrderController
 * @author: HuRui
 * @date: 2024/4/30
 **/
@Controller
@RequestMapping(value="/")
@Api(value="订单业务")
public class OrderController {

    Logger log = LoggerFactory.getLogger(OrderController.class);
    @Resource
    private UserCenter userCenter;
    @Resource
    private BsOrderService orderService;

    @RequestMapping(value="/create",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "创建订单")
    public ResponseData create(@RequestBody OrderModel body) {
        try {
            // 用户session
            UserSessionObject userSession = userCenter.getSessionModel(UserSessionObject.class);
            if (userSession == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_LOGIN_NOT, "");
            }
            if (!userSession.getUser().getStatus().equals(UserStatusEnum.status1.getCode())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "该账户已被禁用，无法进行交易");
            }
            if (body == null || body.getOrderChildList().isEmpty()) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 子订单必填项校验
            for (OrderChildModel orderChild : body.getOrderChildList()) {
                if (orderChild.getProductType() == null
                    || orderChild.getProductId() == null
                    || orderChild.getProductCount() == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
                }

                if (orderChild.getProductType().equals(OrderChildProductTypeEnum.type1.getCode())
                    && orderChild.getProductSpecId() == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
                }
            }

            body.setUserId(userSession.getUser().getId());
            body.setUserName(userSession.getUser().getName());
            body.setUserPhone(userSession.getUser().getPhone());

            return ResponseMsgUtil.success(orderService.create(body));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/cancel",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "取消订单")
    public ResponseData cancel(@RequestBody JSONObject body) {
        try {
            if (body == null|| StringUtils.isBlank(body.getString("orderNo"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            return ResponseMsgUtil.success(orderService.cancel(body.getString("orderNo")));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryOrder",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询订单")
    public ResponseData queryOrder(@RequestParam(value = "orderNo" , required = true) String orderNo) {
        try {

            return ResponseMsgUtil.success(orderService.getDetail(orderNo));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryUserOrder",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询用户订单")
    public ResponseData queryUserOrder(@RequestParam(value = "orderNo" , required = false) String orderNo,
                                       @RequestParam(value = "pageNum" , required = true) Integer pageNum,
                                       @RequestParam(value = "pageSize" , required = true) Integer pageSize) {
        try {
            // 用户session
            UserSessionObject userSession = userCenter.getSessionModel(UserSessionObject.class);
            if (userSession == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.ACCOUNT_LOGIN_NOT, "");
            }
            Map<String,Object> param = new HashMap<>();
            param.put("userId", userSession.getUser().getId());
            param.put("orderNo", orderNo);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(null);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryOrderList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询交易订单列表")
    public ResponseData queryOrderList(@RequestParam(value = "orderNo" , required = false) String orderNo,
                                       @RequestParam(value = "userPhone" , required = false) String userPhone,
                                       @RequestParam(value = "payChannel" , required = false) Integer payChannel,
                                       @RequestParam(value = "payType" , required = false) Integer payType,
                                       @RequestParam(value = "createTimeS" , required = false) Long createTimeS,
                                       @RequestParam(value = "createTimeE" , required = false) Long createTimeE,
                                       @RequestParam(value = "payTimeS" , required = false) Long payTimeS,
                                       @RequestParam(value = "payTimeE" , required = false) Long payTimeE,
                                       @RequestParam(value = "finishTimeS" , required = false) Long finishTimeS,
                                       @RequestParam(value = "finishTimeE" , required = false) Long finishTimeE,
                                       @RequestParam(value = "cancelTimeS" , required = false) Long cancelTimeS,
                                       @RequestParam(value = "cancelTimeE" , required = false) Long cancelTimeE,
                                       @RequestParam(value = "pageNum" , required = true) Integer pageNum,
                                       @RequestParam(value = "pageSize" , required = true) Integer pageSize) {
        try {
            Map<String,Object> param = new HashMap<>();
            param.put("orderNo", orderNo);
            param.put("userPhone", userPhone);
            param.put("payChannel", payChannel);
            param.put("payType", payType);
            param.put("createTimeS", createTimeS);
            param.put("createTimeE", createTimeE);
            param.put("payTimeS", payTimeS);
            param.put("payTimeE", payTimeE);
            param.put("finishTimeS", finishTimeS);
            param.put("finishTimeE", finishTimeE);
            param.put("cancelTimeS", cancelTimeS);
            param.put("cancelTimeE", cancelTimeE);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(orderService.getOrderList(param)));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }



}
