package com.hfkj.service.order;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.RandomUtils;
import com.hfkj.common.utils.RedisUtil;
import com.hfkj.entity.*;
import com.hfkj.model.order.OrderChildModel;
import com.hfkj.model.order.OrderModel;
import com.hfkj.service.coupon.BsOrderCouponNoService;
import com.hfkj.service.coupon.BsOrderCouponService;
import com.hfkj.service.coupon.channel.HuiLianTongCouponService;
import com.hfkj.service.goods.GoodsMsgService;
import com.hfkj.service.goods.GoodsSpecsService;
import com.hfkj.service.goods.GoodsVpdService;
import com.hfkj.service.hlt.HuiLianTongUnionCardService;
import com.hfkj.sysenum.GoodsVpdSourceEnum;
import com.hfkj.sysenum.order.OrderChildProductTypeEnum;
import com.hfkj.sysenum.order.OrderChildStatusEnum;
import com.hfkj.sysenum.order.OrderCouponNoStatusEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 创建订单业务
 * @className: OrderCreateService
 * @author: HuRui
 * @date: 2024/5/7
 **/
@Component
public class OrderCreateService {
    Logger log = LoggerFactory.getLogger(OrderCreateService.class);
    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private BsOrderService orderService;
    @Resource
    private BsOrderCouponService orderCouponService;
    @Resource
    private BsOrderCouponNoService orderCouponNoService;
    @Resource
    private BsOrderChildService orderChildService;
    @Resource
    private GoodsMsgService goodsMsgService;
    @Resource
    private GoodsSpecsService goodsSpecsService;
    @Resource
    private GoodsVpdService goodsVpdService;

    /**
     * 事务产品
     * @param orderChild 子订单
     * @throws Exception
     */
    public OrderChildModel product(OrderModel order, OrderChildModel orderChild) throws Exception {
        // 查询商品
        GoodsMsg goodsMsg = goodsMsgService.queryDetail(orderChild.getProductId());
        if (goodsMsg == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的商品");
        }
        if (goodsMsg.getStatus() != 1) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, goodsMsg.getTitle()+"暂无法就进行购买");
        }
        // 查询商品规格
        GoodsSpecs goodsSpecs = goodsSpecsService.queryDetail(orderChild.getProductSpecId());
        if (goodsSpecs == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的商品规格");
        }
        orderChild.setMerId(goodsMsg.getMerId());
        orderChild.setProductName(goodsMsg.getTitle());
        orderChild.setProductImg(goodsMsg.getListImg());
        orderChild.setProductSpecName(goodsSpecs.getName());
        orderChild.setProductPrice(goodsSpecs.getSalePrice());

        if (goodsMsg.getType() == 1) {
            // TODO 实物


        } else if (goodsMsg.getType() == 2) {
            // 虚拟
            // 查询商品虚拟信息
            GoodsVpd goodsVpd = goodsVpdService.queryDetailBySpecsId(goodsSpecs.getId());
            if (goodsVpd == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的商品虚拟规格");
            }

            // 创建卡券订单
            BsOrderCoupon orderCoupon = new BsOrderCoupon();
            orderCoupon.setOrderNo(orderChild.getOrderNo());
            orderCoupon.setChildOrderNo(orderChild.getChildOrderNo());
            orderCoupon.setMerId(orderChild.getMerId());
            orderCoupon.setUserId(order.getUserId());
            orderCoupon.setUserPhone(order.getUserPhone());
            orderCoupon.setGoodsId(goodsMsg.getId());
            orderCoupon.setGoodsName(goodsMsg.getTitle());
            orderCoupon.setGoodsTypeId(goodsMsg.getGoodsType());
            orderCoupon.setGoodsTypeName(goodsMsg.getGoodsTypeName());
            orderCoupon.setGoodsBrandId(goodsMsg.getGoodsBrand());
            orderCoupon.setGoodsBrandName(goodsMsg.getGoodsBrandName());
            orderCoupon.setGoodsSpecsId(goodsSpecs.getId());
            orderCoupon.setGoodsSpecsName(goodsSpecs.getName());
            orderCoupon.setGoodsSpescOriginalPrice(goodsSpecs.getOriginalPrice());
            orderCoupon.setGoodsVpdId(goodsVpd.getId());
            orderCoupon.setGoodsVpdKey(goodsVpd.getKey());
            orderCoupon.setGoodsVpdSource(goodsVpd.getSource());
            orderCoupon.setGoodsCount(orderChild.getProductCount());
            orderCoupon.setStatus(OrderChildStatusEnum.status1.getCode());
            orderCouponService.editData(orderCoupon);

            for (int i = 0; i < orderCoupon.getGoodsCount(); i++) {
                // 卡券订单卡密
                BsOrderCouponNo orderCouponNo = new BsOrderCouponNo();
                orderCouponNo.setCouponOrderId(orderCoupon.getId());
                orderCouponNo.setOrderNo(orderCoupon.getId() + RandomUtils.number(6, false));
                orderCouponNo.setExpireStatus(false);
                orderCouponNo.setExceptionStatus(false);
                orderCouponNo.setStatus(OrderCouponNoStatusEnum.status1.getCode());

                if (goodsVpd.getSource().equals(GoodsVpdSourceEnum.type4.getCode())) {
                    // 下单请求
                    JSONObject jsonObject = HuiLianTongCouponService.recharge(orderCouponNo.getOrderNo(),
                            1,
                            orderCoupon.getGoodsVpdKey(),
                            orderCoupon.getUserPhone());
                    if (!jsonObject.getString("respCode").equals("0000")) {
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "下单失败");
                    }
                    JSONObject response = HuiLianTongUnionCardService.resolveResponse(jsonObject.getString("data"));
                    if (!response.getString("respCode").equals("0000")) {
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, response.getString("msg"));
                    }
                    orderCouponNo.setChannelOrderNo(response.getJSONObject("data").getString("rechargeOrderId"));
                }
                orderCouponNoService.editData(orderCouponNo);
            }
        }
        return orderChild;
    }


}
